(* direktes Ausdrucken von ASCII-Text-Dateien
   J. Rathlev, Feb. 2000 *)

unit Prtmain;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Printers, Dialogs, StdCtrls, HListBox, Buttons, ExtCtrls, Spin,
  Menus, RandDlg, MsgWind, JrUtils, XPMan;

const
  Prog = 'Print ASCII-Text';
  Vers = 'Vers. 1.3';
  CopRgt = '(c) 2006 J. Rathlev, IEAP, Uni-Kiel';

  DefHRand : TFRect =(Left: 2.0; Top: 1.5; Right: 1.0; Bottom: 1.5);
  DefQRand : TFRect =(Left: 1.5; Top: 2.0; Right: 1.5; Bottom: 1.0);
  DefTopPos = 0.0;  (* Abstand obere Papierkante - Nullpunkt *)
  DefLineDist = 1.0;   (* Zeilenabstand *)
  PtToCm = 0.0353;   (* 1 Pt in cm *)
  HdLine = 1.5;      (* Zeilenhhe der Kopfzeile *)

  DefTabSpace = 8;
  FMax = 1000;

  SkipChar : set of char = [' ',#$A,#$D];

type
  FileListTyp = array [1..FMax] of string;

  THauptForm = class(TForm)
    Label1: TLabel;
    Label2: TLabel;
    VerzWahlBtn: TSpeedButton;
    FileBtn: TSpeedButton;
    LayoutRG: TRadioGroup;
    EndeBtn: TBitBtn;
    VerzEdit: THistoryCombo;
    FileEdit: THistoryCombo;
    OpenDialog: TOpenDialog;
    InfoBtn: TSpeedButton;
    EinstellBtn: TSpeedButton;
    FormatGB: TGroupBox;
    RBH: TRadioButton;
    RBQ: TRadioButton;
    Panel1: TPanel;
    HochImg: TImage;
    QuerImg: TImage;
    RandBtn: TSpeedButton;
    FontBtn: TSpeedButton;
    PrinterSetupDialog: TPrinterSetupDialog;
    FontDialog: TFontDialog;
    UmlautRG: TRadioGroup;
    DruckBtn: TSpeedButton;
    Panel2: TPanel;
    Drucker: TEdit;
    PFont: TEdit;
    TabBtn: TSpeedButton;
    btnFilter: TSpeedButton;
    XPManifest1: TXPManifest;
    procedure VerzWahlBtnClick(Sender: TObject);
    procedure DruckBtnClick(Sender: TObject);
    procedure EndeBtnClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure InfoBtnClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure ShowFormat;
    procedure FileBtnClick(Sender: TObject);
    procedure VerzEditExit(Sender: TObject);
    procedure FileEditExit(Sender: TObject);
    procedure EinstellBtnClick(Sender: TObject);
    procedure VerzEditChange(Sender: TObject);
    procedure RandBtnClick(Sender: TObject);
    procedure FontBtnClick(Sender: TObject);
    procedure GetPaperSize (Printer          : TPrinter;
                            var Width,Height : double);
    procedure RBHClick(Sender: TObject);
    procedure TabBtnClick(Sender: TObject);
    procedure btnFilterClick(Sender: TObject);
  private
    { Private-Deklarationen }
    PrgPfad,PrgName,
    IniName          : string;
    PrtOri      : TPrinterOrientation;
    TabSpace,
    PrtNdx      : integer;
    TextFont    : TFont;
    HRand,QRand : TFRect;  (* Rnder in cm *)
    PrSize      : TFPoint; (* Papiergre in cm einschl. Rand *)
    TopPos      : double;
    PrtAbort    : boolean;
    TxtFilter   : string;

  public
    { Public-Deklarationen }
    (* Text drucken *)
    procedure PrintText (var fs      : TFileStream;
                         Name        : string;
                         Header      : integer;
                         Size        : TFPoint;
                         Rand        : TFRect;
                         LineDist    : double);
  end;

var
  HauptForm: THauptForm;

implementation

uses Filtdlg, IniFiles, WinDesktop, ShlObj, InpNumber, ShellDirDlg;

{$R *.DFM}

const
  IniExt = 'ini';

  (* INI-Sektionen *)
  CfGSekt = 'Config';
  VerzSekt = 'Directories';
  FileSekt = 'Files';
  HochSekt = 'Portrait';
  QuerSekt = 'Landscape';

  (* INI-Variablen *)
  IniPrtIndex = 'PrtIndex';
  IniOrientation = 'Orientation';
  IniLayout = 'Layout';
  IniUmlaut = 'Charset';
  IniFont = 'Font';
  IniFontSize = 'FontSize';
  IniLeftMarg = 'LeftMargin';
  IniRightMarg = 'RightMargin';
  IniTopMarg = 'TopMargin';
  IniBottomMarg = 'BottomMargin';
  IniTopLine = 'TopLine';
  IniTabSpace = 'TabSpace';
  iniFilter = 'Filter';

{ ------------------------------------------------------------------- }
(* INI-File lesen *)
procedure THauptForm.FormCreate(Sender: TObject);
var
  IniFile  : TIniFile;
  i        : integer;
begin
  TextFont:=TFont.Create;
  PrgPfad:=ExtractFilePath(Application.ExeName);
  PrgName:=ExtractFileName(DelExt(Application.ExeName));
  IniName:=Erweiter(GetDesktopFolder(CSIDL_APPDATA),
    ExtractFileName(DelExt(Application.ExeName)),IniExt);
  IniFile:=TIniFile.Create(IniName);
  with IniFile do begin
    TextFont.Name:=ReadString(CfgSekt,iniFont,'Courier New');
    TextFont.Size:=ReadInteger(CfgSekt,iniFontSize,10);
    i:=ReadInteger(CfgSekt,IniUmlaut,0);
    if (i<0) or (i>=UmlautRG.Items.Count) then i:=0;
    UmlautRG.ItemIndex:=i;
    TopPos:=ReadFloat (CfgSekt,IniTopLine,DefTopPos);
    TabSpace:=ReadInteger (CfgSekt,IniTabSpace,DefTabSpace);
    TxtFilter:=ReadString(CfgSekt,IniFilter,'Text-Dateien|*.txt|alle|*.*');
    i:=ReadInteger(CfgSekt,IniLayout,1);
    if (i<0) or (i>=LayoutRG.Items.Count) then i:=1;
    LayoutRG.ItemIndex:=i;
    PrtNdx:=ReadInteger (CfgSekt,IniPrtIndex,-1);
    i:=ReadInteger (CfgSekt,IniOrientation,ord(poLandscape));
    if (i<0) or (i>1) then i:=1;
    PrtOri:=TPrinterOrientation(i);
    with HRand do begin
      Left:=ReadFloat (HochSekt,IniLeftMarg,DefHRand.Left);
      Right:=ReadFloat (HochSekt,IniRightMarg,DefHRand.Right);
      Top:=ReadFloat (HochSekt,IniTopMarg,DefHRand.Top);
      Bottom:=ReadFloat (HochSekt,IniBottomMarg,DefHRand.Bottom);
      end;
    with QRand do begin
      Left:=ReadFloat (QuerSekt,IniLeftMarg,DefQRand.Left);
      Right:=ReadFloat (QuerSekt,IniRightMarg,DefQRand.Right);
      Top:=ReadFloat (QuerSekt,IniTopMarg,DefQRand.Top);
      Bottom:=ReadFloat (QuerSekt,IniBottomMarg,DefQRand.Bottom);
      end;
    Free;
    end;
  VerzEdit.LoadFromIni (IniName,VerzSekt);
  FileEdit.LoadFromIni (IniName,FileSekt);
  end;

{ ------------------------------------------------------------------- }
(* INI-File schreiben *)
procedure THauptForm.FormDestroy(Sender: TObject);
var
  IniFile : TIniFile;
begin
  IniFile:=TIniFile.Create(IniName);
  with IniFile do begin
    WriteString(CfgSekt,IniFont,TextFont.Name);
    WriteInteger(CfgSekt,iniFontSize,TextFont.Size);
    WriteInteger(CfgSekt,IniUmlaut,UmlautRG.ItemIndex);
    WriteString (CfgSekt,IniTopLine,FloatToStrF(TopPos,ffFixed,5,2));
    WriteInteger (CfgSekt,IniTabSpace,TabSpace);
    WriteString(CfgSekt,IniFilter,TxtFilter);
    WriteInteger(CfgSekt,IniLayout,LayoutRG.ItemIndex);
    WriteInteger (CfgSekt,IniPrtIndex,PrtNdx);
    WriteInteger (CfgSekt,IniOrientation,ord(PrtOri));
    with HRand do begin
      WriteString (HochSekt,IniLeftMarg,FloatToStrF(Left,ffFixed,5,2));
      WriteString (HochSekt,IniRightMarg,FloatToStrF(Right,ffFixed,5,2));
      WriteString (HochSekt,IniTopMarg,FloatToStrF(Top,ffFixed,5,2));
      WriteString (HochSekt,IniBottomMarg,FloatToStrF(Bottom,ffFixed,5,2));
      end;
    with QRand do begin
      WriteString (QuerSekt,IniLeftMarg,FloatToStrF(Left,ffFixed,5,2));
      WriteString (QuerSekt,IniRightMarg,FloatToStrF(Right,ffFixed,5,2));
      WriteString (QuerSekt,IniTopMarg,FloatToStrF(Top,ffFixed,5,2));
      WriteString (QuerSekt,IniBottomMarg,FloatToStrF(Bottom,ffFixed,5,2));
      end;
    Free;
    end;
  end;

{ ------------------------------------------------------------------- }
(* Programmende *)
procedure THauptForm.EndeBtnClick(Sender: TObject);
begin
  Close;
end;

{ ------------------------------------------------------------------- }
(* Format-Radio-Buttons anzeigen *)
procedure THauptForm.ShowFormat;
begin
  if PrtOri=poLandscape then RBQ.Checked:=true
  else RBH.Checked:=true;;
  end;

{-----------------------------------------------------------------------}
(* Papiergre in cm ermitteln *)
procedure THauptForm.GetPaperSize (Printer          : TPrinter;
                                   var Width,Height : double);
var
  w,h : integer;
begin
  with Printer do begin
    w:=GetDeviceCaps (Handle,HORZSIZE);
    h:=GetDeviceCaps (Handle,VERTSIZE);
    if w<=0 then Width:=28 else Width:=w/10.0;
    if h<=0 then Height:=20 else Height:=h/10.0;
    end;
  end;

{ ------------------------------------------------------------------- }
(* Formular anzeigen *)
procedure THauptForm.FormShow(Sender: TObject);
begin
  with Printer do begin
    Orientation:=PrtOri;
    if (PrtNdx<-1) or (PrtNdx>=Printers.Count) then PrtNdx:=-1;
    PrinterIndex:=PrtNdx;
    if PrtNdx=-1 then PrtNdx:=PrinterIndex;
    if Printers.Count>0 then DruckBtn.Enabled:=true
    else DruckBtn.Enabled:=false;
    end;
  with PrSize do GetPaperSize (Printer,X,Y);
  with VerzEdit do if Items.Count>0 then Text:=Items[0];
  with FileEdit do if Items.Count>0 then Text:=Items[0];
  ShowFormat;
  Drucker.Text:=Printer.Printers[PrtNdx];
  with TextFont do PFont.Text:=Name+' '+IntToStr(Size)+' pt';
  end;

{ ------------------------------------------------------------------- }
(* Verzeichnis whlen *)
procedure THauptForm.VerzWahlBtnClick(Sender: TObject);
var
  s : string;
begin
  s:=VerzEdit.Text;
  if ShellDirDialog.Execute ('Verzeichnis fr Text-Dateien',true,'',s) then begin
    VerzEdit.Text:=s;
    VerzEdit.AddItem(s);
    with FileEdit do begin
      Text:='*.txt'; AddItem(Text);
      end;
    end;
  end;

{ ------------------------------------------------------------------- }
(* Text drucken *)
procedure THauptForm.PrintText (var fs      : TFileStream;
                                Name        : string;
                                Header      : integer;
                                Size        : TFPoint;
                                Rand        : TFRect;
                                LineDist    : double);
var
  x,y,dx,dy,h,
  i,j,k,n       : Integer;
  lp            : word;
  f             : double;
  del           : char;
  line,
  s,sp          : string;
  np            : boolean;
  fl            : longint;

  function ReadNxtWord(s : string; var k : integer) :string;
  var
    m,n : integer;
  begin
    m:=length(s); n:=k;
    while (n<=m) and not (s[n] in SkipChar)do inc(n);
    while (n<=m) and (s[n] in SkipChar) do inc(n);
    Result:=copy(s,k,n-k); k:=n;
    end;

  procedure CheckForNewPage;
  begin
    with Printer do begin
      with Canvas do begin
        if (y>dy) or (Del=#12) then begin
          NewPage; y:=round((Rand.Top)*100);
          TextOut (0,0,'');
          SetMapMode (Canvas.Handle,MM_LOMETRIC);
          np:=true;
          end;
        if np then begin
          np:=false;
          if Header=2 then begin
            SetTextAlign (Canvas.Handle,TA_CENTER);
            s:='-'+IntToStr(PageNumber)+'-';
            TextOut(x+dx div 2,-y,s);
            end;
          if Header=1 then begin
            s:='***   '+StripPath(Name,50)+' - '+DateToStr(Date);
            TextOut (x,-y,s);
            SetTextAlign (Canvas.Handle,TA_RIGHT);
            s:='Seite '+IntToStr(PageNumber)+'   ***';
            TextOut (x+dx,-y,s);
            MoveTo (x,-y-h-2); LineTo (x+dx,-y-h-2);
            end;
          SetTextAlign (Canvas.Handle,TA_LEFT);
          y:=y+round(HdLine*h/LineDist);
          end;
        end;
      end;    
    end;

  procedure ReadStreamLine (fs : TStream; var line : string; var delim : char);
  var
    c : char;
  begin
    line:=''; delim:=#0;
    with fs do if Position<Size then begin
      repeat
        Read(c,1);
        if c>=#32 then line:=line+c;
        until (c=#13) or (c=#12) or (Position>=Size);
      if (Position<Size) then delim:=c;
      end
    end;

begin
  with Printer do begin
    with Canvas do begin
      TextOut (0,0,'');
      SetMapMode (Handle,MM_LOMETRIC);
      y:=round((Rand.Top)*100);
      x:=round((Rand.Left)*100);
      dy:=round((Size.Y-Rand.Bottom)*100);
      dx:=round((Size.X-Rand.Right)*100)-x;
      f:=PtToCm*100*TextFont.Size;    (* Punkte in 0,1 mm umrechnen *)
      Font:=TextFont;
      Font.Height:=round(f);
      h:=round(LineDist*f);                  (* Zeilenabstand *)
      np:=true;
      fl:=fs.Size;
      while (fs.Position<fs.Size) do begin
        ReadStreamLine(fs,s,del);
        (* Tabs auswerten *)
        line:='';
        for i:=1 to length(s) do begin
          if s[i]=#9 then begin
            k:=length(line);
            for j:=1 to TabSpace-(k mod TabSpace) do line:=line+' '
            end
          else line:=line+s[i];
          end;
        if UmlautRG.ItemIndex=1 then line:=StrToANSI(line);
        Hinweis.ShowProgress (round(100.0*fs.Position/fl));
        Application.ProcessMessages;
        sp:='';
        // Zeilenumbruch bei langen Zeilen
        if TextWidth(line)>dx then begin
          n:=1;
          while (n<=length(line)) and (line[n] in SkipChar)do inc(n);
          if n<=length(line) then begin
            dec(n);
            sp:=copy(line,1,n); // Einrcken
            delete (line,1,n);
            repeat
              s:=''; n:=1;
              repeat
                k:=n-1;
                s:=s+ReadNxtWord(line,n);
                until (TextWidth(s)>dx);
              CheckForNewPage;
              TextOut (x,-y,sp+CutSp(copy(line,1,k)));
              y:=y+h;
              delete (line,1,k);
              until TextWidth(line)<=dx;
            end;
          end;
        CheckForNewPage;
        TextOut (x,-y,sp+line);
        y:=y+h;
        end;
      end;
    end;
  end;

{ ------------------------------------------------------------------- }
(* Text ausdrucken *)
procedure THauptForm.DruckBtnClick(Sender: TObject);
var
  ARand         : TFrect;
  FLPtr         : ^FileListTyp;
  DMaske        : string;
  DirInfo       : TSearchRec;
  FCnt,i,
  DosError      : integer;
  s,t,
  FName         : string;
  InText        : TFileStream;
  x,y           : integer;
begin
  x:=Left+50; y:=Top+50;
  DruckBtn.Down:=true;
  if DruckBtn.Down then begin
    PrtAbort:=false;
    if PrtOri=poLandscape then begin
      ARand:=QRand;
      with ARand do begin
        Right:=Right-TopPos; Left:=Left+TopPos;
        end;
      end
    else begin
      ARand:=HRand;
      with ARand do begin
        Top:=Top-TopPos; Bottom:=Bottom+TopPos;
        end;
      end;
    t:=FileEdit.Text;
    if length(t)>0 then begin
      repeat
        s:=ReadNxtStr (t,';');
        DMaske:=Erweiter(VerzEdit.Text,s,'');
        New (FLPtr); FCnt:=0;
        DosError:=FindFirst (DMaske,faArchive,DirInfo);
        while DosError=0 do begin
          inc (FCnt);
          FLPtr^[FCnt]:=DirInfo.Name;
          DosError:=FindNext (DirInfo);
          end;
        Sysutils.FindClose (DirInfo);
        if FCnt>0 then for i:=1 to FCnt do begin
          FName:=Erweiter(VerzEdit.Text,FLPtr^[i],'');
          Printer.Title:=FName;
          Hinweis.ShowStringPos (FName,0,taCenter,0,'Datei:','wird gedruckt!',
                  'Textdruck','auf '+Drucker.Text,x,y);
          Application.ProcessMessages;
          if FileExists(FName) then begin
            InText:=TFileStream.Create(FName,fmOpenRead);
            with Printer do begin
              BeginDoc;
              PrintText (InText,FName,LayoutRG.ItemIndex,PrSize,ARand,DefLineDist);
              EndDoc;
              InText.Free;
              end
            end
          else ErrorDialog('','Datei:'+FName+#13' nicht gefunden!',Point(x,y));
          end
        else ErrorDialog('','Keine passenden Dateien zu'#13+t+' gefunden!',Point(x,y));
        Hinweis.Close;
        Dispose (FLPtr);
        until length(t)=0;
      end;
    DruckBtn.Down:=false;
    end
  else PrtAbort:=true;
  end;

{ ------------------------------------------------------------------- }
procedure THauptForm.InfoBtnClick(Sender: TObject);
begin
  InfoDialog('',Prog+Vers+#13+CopRgt+#13'E-Mail: rathlev@physik.uni-kiel.de');
  end;

{ ------------------------------------------------------------------- }
(* Datei auswhlen *)
procedure THauptForm.FileBtnClick(Sender: TObject);
var
  i : integer;
  s : string;
begin
  with OpenDialog do begin
    InitialDir:=VerzEdit.Text;
    Filename:='';
    Filter:=TxtFilter;
    Title:='Text-Dateien auswhlen';
    if Execute then begin
      s:=ExtractFilePath(Files.Strings[0]);
      if s<>VerzEdit.Text then begin
        VerzEdit.Text:=s;
        VerzEdit.AddItem(s);
        end;
      with Files do begin
        s:='';
        for i:=0 to Count-1 do s:=s+ExtractFilename(Strings[i])+' ';
        end;
      delete (s,length(s),1);
      with FileEdit do begin
        Text:=s; AddItem(s);
        end;
      end;
    end;
  end;

{ ------------------------------------------------------------------- }
(* Verzeichnis auswhlen *)
procedure THauptForm.VerzEditExit(Sender: TObject);
begin
  with VerzEdit do AddItem(Text);
  end;

{ ------------------------------------------------------------------- }
procedure THauptForm.FileEditExit(Sender: TObject);
begin
  with FileEdit do AddItem(Text);
end;

procedure THauptForm.EinstellBtnClick(Sender: TObject);
begin
  if not DruckBtn.Down then begin
    PrinterSetupDialog.Execute;
    with Printer do begin
      PrtOri:=Orientation;
      PrtNdx:=PrinterIndex;
      ShowFormat;
      Drucker.Text:=Printer.Printers[PrtNdx];
      end;
    end;
  end;

procedure THauptForm.VerzEditChange(Sender: TObject);
begin
  with FileEdit do begin
    Text:='*.txt'; AddItem(Text);
    end;
end;

procedure THauptForm.RandBtnClick(Sender: TObject);
var
  ARand : TFRect;
begin
  if PrtOri=poLandscape then ARand:=QRand
  else ARand:=HRand;
  if RandDialog.Execute (Printer.Printers[PrtNdx],true,ARand,TopPos) then begin
    if PrtOri=poLandscape then QRand:=ARand
    else HRand:=ARand;
    end;
  end;

procedure THauptForm.FontBtnClick(Sender: TObject);
begin
  with FontDialog do begin
    Font:=TextFont;
    if Execute then begin
      TextFont:=Font;
      with TextFont do PFont.Text:=Name+' '+IntToStr(Size)+' pt';
      end;
    end;
  end;

procedure THauptForm.RBHClick(Sender: TObject);
begin
  if RBQ.Checked then PrtOri:=PoLandscape
  else PrtOri:=poPortrait;
  with Printer do begin
    PrinterIndex:=PrtNdx; Orientation:=PrtOri;
    end;
  with PrSize do GetPaperSize (Printer,X,Y);
  end;

procedure THauptForm.TabBtnClick(Sender: TObject);
var
  n : integer;
begin
  n:=TabSpace;
  if InputInteger(CenterPos,'Tabweite','Anzahl der Zeichen:','','Abbrechen',
      1,2,1,100,false,n) then TabSpace:=n;
  end;

procedure THauptForm.btnFilterClick(Sender: TObject);
begin
  FilterDialog.Execute ('Dateitypen',TxtFilter);
  end;

end.
